export class Pagination<TItem> {
  public currentPage: TItem[] = [];
  public currentPageNumber: number = 1;
  public items: TItem[];
  public limit: number;
  public onPageChange: Function = () => {};
  public pageCount: number[];
  public serverSide: boolean = false;

  /**
   * PaginationService object
   * @param  {TItem[] = []}          items an array of items
   * @param  {number  = 10}          limit
   * @return {PaginationService}
   */
  constructor(items: TItem[] = [], limit: number = 10) {
    this.items = items;
    this.limit = limit;
    // init
    this.getPagesCount();
    this.getPageFirst();
  }

  /**
   * returns if the page is in the pages count
   * @param  {number}   pageNumber [description]
   * @param  {number[]} pageCount  [description]
   * @return {boolean}             [description]
   */
  public static isInRange(pageNumber: number, pageCount: number[]): boolean {
    return pageCount.filter(page => page === pageNumber).length > 0;
  }

  /**
   * gets pages count
   * @param  {number}   itemsLength
   * @param  {number}   limit
   * @return {number[]}
   */
  public static getPagesCount(itemsLength: number, limit: number): number[] {
    let count = Math.floor(itemsLength / limit) + (itemsLength % limit > 0 ? 1 : 0);
    let pages = [];

    while (pages.length < count) {
      pages.push(pages.length + 1);
    }
    return pages;
  }

  /**
   * returns if is the current page
   * @param  {number}  pageNumber
   * @return {boolean}
   */
  public isCurrent(pageNumber: number): boolean {
    return pageNumber === this.currentPageNumber;
  }

  /**
   * returns if the page requested is in range
   * @param  {number}  pageNumber
   * @return {boolean}
   */
  public isInRange(pageNumber: number): boolean {
    return Pagination.isInRange(pageNumber, this.pageCount);
  }

  /**
   * gets page number
   * @param  {number}  pageNumber
   * @return {TItem[]}
   */
  public getPage(pageNumber: number): TItem[] {
    this.currentPageNumber = pageNumber;
    let page = []
    let limitStart = this.limit * (this.currentPageNumber - 1);
    let limitStop = this.limit * (this.currentPageNumber);
    page = this.items.filter((item, index) => index >= limitStart && index < limitStop);
    this.currentPage = page;
    this.onPageChange.call(this, pageNumber);
    return page;
  }

  /**
   * gets first page
   * @return {TItem[]}
   */
  public getPageFirst(): TItem[] {
    return this.getPage(1);
  }

  /**
   * gets last page
   * @return {TItem[]}
   */
  public getPageLast(): TItem[] {
    return this.getPage(this.pageCount[this.pageCount.length - 1]);
  }

  /**
   * returns next page
   * @return {TItem[]}
   */
  public getPageNext(): TItem[] {
    let nextPage = this.currentPageNumber + 1;
    if (this.isInRange(nextPage)) {
      return this.getPage(nextPage);
    } else {
      return this.currentPage;
    }
  }

  /**
   * returns prev page
   * @return {TItem[]}
   */
  public getPagePrev(): TItem[] {
    let prevPage = this.currentPageNumber - 1;
    if (prevPage > 0) {
      return this.getPage(prevPage);
    } else {
      return this.currentPage;
    }
  }

  /**
   * returns pages count
   * @return {number[]}
   */
  public getPagesCount(): number[] {
    this.pageCount = Pagination.getPagesCount(this.items.length, this.limit);
    return this.pageCount;
  }

  /**
   * sets a new limit
   * @param {number} limit [description]
   */
  public setLimit(limit: number): void {
    this.limit = limit;
    this.getPagesCount();
  }

  /**
   * sets remote pages count
   * @param {number} pages
   */
  public setPagesCount(pages: number): void {
    this.pageCount = [];
    while(this.pageCount.length < pages) {
      this.pageCount.push(this.pageCount.length + 1);
    }
  }

  /**
   * used when the service is using a remote pagination API
   * @param {TItem[]} items        the paginated items
   */
  public setPageItems(items: TItem[]): void {
    this.currentPage = items;
  }
}
