import * as React from 'react';

export interface IColProps {
  /**
   * additional css
   * @type {string}
   */
  className?: string;
  /**
   * the column offset property
   *  you can set a negative or positive value in each breakpoint
   * @type {IColBreakpoints}
   */
  offset?: IColBreakpoints;
  /**
   * the column move property
   *  you can set a negative or positive value in each breakpoint
   * @type {IColBreakpoints}
   */
  move?: IColBreakpoints;
  /**
   * the column width property
   * @type {IColBreakpoints}
   */
  width?: IColBreakpoints;
}

export interface IColBreakpoints {
  /**
   * Depending on ride-components xs breakpoint
   * @type {number}
   */
  xs?: number;
  /**
   * Depending on ride-components sm breakpoint
   * @type {number}
   */
  sm?: number;
  /**
   * Depending on ride-components md breakpoint
   * @type {number}
   */
  md?: number;
  /**
   * Depending on ride-components lg breakpoint
   * @type {number}
   */
  lg?: number;
  /**
   * Depending on ride-components xl breakpoint
   * @type {number}
   */
  xl?: number;
}

function cannotBeNegative(_number: number): boolean {
  if (_number && isNegative(_number)) throw new Error('Col width cannot be negative.');
  return true;
}

function isNegative(_number: number): boolean {
  return _number < 0;
}

function getColumnCSS(col: IColBreakpoints): string {
  let _css = ['col'];
  if (!col) return '';
  if (!!col.xs && cannotBeNegative(col.xs)) _css.push(`col-xs-${col.xs}`);
  if (!!col.sm && cannotBeNegative(col.sm)) _css.push(`col-sm-${col.sm}`);
  if (!!col.md && cannotBeNegative(col.md)) _css.push(`col-md-${col.md}`);
  if (!!col.lg && cannotBeNegative(col.lg)) _css.push(`col-lg-${col.lg}`);
  if (!!col.xl && cannotBeNegative(col.xl)) _css.push(`col-xl-${col.xl}`);
  return _css.join(' ');
}

function getMoveCSS(move: IColBreakpoints): string {
  let _css = [];
  if (!move) return '';
  if (move.xs) _css.push(`${ isNegative(move.xs) ? 'pull' : 'push' }-xs-${makePositive(move.xs)}`);
  if (move.sm) _css.push(`${ isNegative(move.sm) ? 'pull' : 'push' }-sm-${makePositive(move.sm)}`);
  if (move.md) _css.push(`${ isNegative(move.md) ? 'pull' : 'push' }-md-${makePositive(move.md)}`);
  if (move.lg) _css.push(`${ isNegative(move.lg) ? 'pull' : 'push' }-lg-${makePositive(move.lg)}`);
  if (move.xl) _css.push(`${ isNegative(move.xl) ? 'pull' : 'push' }-xl-${makePositive(move.xl)}`);
  return _css.join(' ');
}

function getOffsetCSS(offset: IColBreakpoints): string {
  let _css = [];
  if (!offset) return '';
  if (offset.xs) _css.push(`offset-${ isNegative(offset.xs) ? 'left' : 'right' }-xs-${makePositive(offset.xs)}`);
  if (offset.sm) _css.push(`offset-${ isNegative(offset.sm) ? 'left' : 'right' }-sm-${makePositive(offset.sm)}`);
  if (offset.md) _css.push(`offset-${ isNegative(offset.md) ? 'left' : 'right' }-md-${makePositive(offset.md)}`);
  if (offset.lg) _css.push(`offset-${ isNegative(offset.lg) ? 'left' : 'right' }-lg-${makePositive(offset.lg)}`);
  if (offset.xl) _css.push(`offset-${ isNegative(offset.xl) ? 'left' : 'right' }-xl-${makePositive(offset.xl)}`);
  return _css.join(' ');
}

function makePositive(_number: number): number {
  return _number < 0 ? -1 * _number : _number;
}

export class Cl extends React.Component<IColProps, {}> {
  public render() {
    let css: string;
    let _css = [''];
    if (this.props.className) _css.push(this.props.className);
    _css.push(getColumnCSS(this.props.width));
    _css.push(getMoveCSS(this.props.move));
    _css.push(getOffsetCSS(this.props.offset));
    css = _css.join(' ').trim();
    return (
      <div className={css}>{this.props.children}</div>
    )
  }
}
